@@ -113,18 +113,13 @@ void do_warn(SV* h, int rc, char* what) {
  *           been called in the latter case
  *
  **************************************************************************/
-int dbd_db_login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
+
+static int Login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
 		 char* password) {
     char* copy = NULL;
     char* host = NULL;
     char* ptr;
 
-    if (dbis->debug >= 2)
-        fprintf(DBILOGFP, "imp_dbh->connect: dsn = %s, uid = %s, pwd = %s\n",
-	       dbname ? dbname : "NULL",
-	       user ? user : "NULL",
-	       password ? password : "NULL");
-
     /*
      *  dbname may be "db:host" or "db;host"
      */
@@ -141,25 +136,37 @@ int dbd_db_login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
      *  Try to connect
      */
     imp_dbh->svsock = &imp_dbh->mysql;
-    if (!dbd_db_connect(imp_dbh->svsock, host, user, password, dbname)) {
-	DO_ERROR(dbh, JW_ERR_CONNECT, imp_dbh->svsock);
+    if (!dbd_db_connect(imp_dbh->svsock, host, user, password)) {
 	if (copy) free(copy);
 	return FALSE;
     }
 
-#if !defined(DBD_MYSQL)  ||  !defined(MYSQL_VERSION_ID)  ||  (MYSQL_VERSION_ID < 32200)
     /*
      *  Connected, now try to login
      */
     if (MySelectDb(imp_dbh->svsock, dbname)) {
         if (copy) free(copy);
-	DO_ERROR(dbh, JW_ERR_SELECT_DB, imp_dbh->svsock);
 	MyClose(imp_dbh->svsock);
 	return FALSE;
     }
-#endif
 
     if (copy) free(copy);
+    return TRUE;
+}
+
+int dbd_db_login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
+		 char* password) {
+    if (dbis->debug >= 2)
+        fprintf(DBILOGFP, "imp_dbh->connect: dsn = %s, uid = %s, pwd = %s\n",
+	       dbname ? dbname : "NULL",
+	       user ? user : "NULL",
+	       password ? password : "NULL");
+
+    if (!Login(dbh, imp_dbh, dbname, user, password)) {
+	DO_ERROR(dbh, MyErrno(imp_dbh->svsock, JW_ERR_CONNECT),
+		 imp_dbh->svsock);
+	return FALSE;
+    }
 
     /*
      *  Tell DBI, that dbh->disconnect should be called for this handle
@@ -249,10 +256,6 @@ int dbd_db_disconnect(SV* dbh, imp_dbh_t* imp_dbh) {
  **************************************************************************/
 
 int dbd_discon_all (SV *drh, imp_drh_t *imp_drh) {
-#if defined (dTHR)
-    dTHR;
-#endif
-
     /* The disconnect_all concept is flawed and needs more work */
     if (!dirty && !SvTRUE(perl_get_sv("DBI::PERL_ENDING",0))) {
 	sv_setiv(DBIc_ERR(imp_drh), (IV)1);
@@ -366,7 +369,6 @@ SV* dbd_db_FETCH_attrib(SV* dbh, imp_dbh_t* imp_dbh, SV* keysv) {
 	 */
         return &sv_yes;
     }
-        
     if (kl == 5  &&  strEQ(key, "errno")) {
 #if defined(DBD_MYSQL)  &&  defined(mysql_errno)
 	return sv_2mortal(newSViv((IV)mysql_errno(imp_dbh->svsock)));
@@ -376,11 +378,6 @@ SV* dbd_db_FETCH_attrib(SV* dbh, imp_dbh_t* imp_dbh, SV* keysv) {
     } else if (kl == 6  &&  strEQ(key, "errmsg")) {
 	char* msg = MyError(imp_dbh->svsock);
 	return sv_2mortal(newSVpv(msg, strlen(msg)));
-    } else if (kl == 4  &&  strEQ(key, "info")) {
-        char* info = mysql_info(imp_dbh->svsock);
-        return info ? sv_2mortal(newSVpv(info, strlen(info))) : &sv_undef;
-    } else if (kl == 9  &&  strEQ(key, "thread_id")) {
-        return sv_2mortal(newSViv(mysql_thread_id(imp_dbh->svsock)));
     }
     return Nullsv;
 }
@@ -477,8 +474,6 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 	}
     }
 
-    if (*cdaPtr) { MyFreeResult(*cdaPtr); *cdaPtr = NULL; }
-
     if (slen >= 10
 	&&  tolower(sbuf[0]) == 'l'
 	&&  tolower(sbuf[1]) == 'i'
@@ -516,7 +511,9 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 
 	return 0;
     } else {
-	if (MyQuery(svsock, sbuf, slen) == -1) {
+	if ((MyQuery(svsock, sbuf, slen) == -1)  &&
+	    (!MyReconnect(svsock, h)
+	     ||	 (MyQuery(svsock, sbuf, slen) == -1))) {
 	    Safefree(salloc);
 	    DO_ERROR(h, JW_ERR_QUERY, svsock);
 	    return -2;
@@ -553,9 +550,6 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
     D_imp_dbh_from_sth;
     SV** statement;
     int i;
-#if defined (dTHR)
-    dTHR;
-#endif
 
     if (dbis->debug >= 2) {
         fprintf(DBILOGFP, "    -> dbd_st_execute for %08lx\n", (u_long) sth);
@@ -676,6 +670,10 @@ AV* dbd_st_fetch(SV* sth, imp_sth_t* imp_sth) {
 	if (col) {
 	    STRLEN len = lengths[i];
 	    if (ChopBlanks) {
+		while(len && isspace(*col)) {
+		    ++col;
+		    --len;
+		}
 		while(len && isspace(col[len-1])) {
 		    --len;
 		}
@@ -713,10 +711,6 @@ AV* dbd_st_fetch(SV* sth, imp_sth_t* imp_sth) {
  **************************************************************************/
 
 int dbd_st_finish(SV* sth, imp_sth_t* imp_sth) {
-#if defined (dTHR)
-    dTHR;
-#endif
-
     /* Cancel further fetches from this cursor.                 */
     /* We don't close the cursor till DESTROY.                  */
     /* The application may re execute it.                       */
@@ -957,9 +951,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 	 *  dbd_st_destroy and dbd_st_execute.
 	 */
 	if (cacheit) {
-	    imp_sth->av_attr[what] = av;
-	} else {
-	    return sv_2mortal(newRV_noinc((SV*) av));
+	    imp_sth->av_attr[what] = (AV*) SvREFCNT_inc((SV*)av);
 	}
     }
 
@@ -1099,9 +1091,6 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	}
 	break;
       case 'r':
-	/*
-	 * Deprecated, use 'result'
-	 */
 	if (strEQ(key, "result")) {
 	    retsv = sv_2mortal(newSViv((IV) imp_sth->cda));
 	}
@@ -1209,3 +1198,57 @@ int dbd_bind_ph (SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 
     return BindParam(&imp_sth->params[paramNum - 1], value, sql_type);
 }
+
+
+
+/***************************************************************************
+ *
+ *  Name:    MysqlReconnect
+ *
+ *  Purpose: If the server has disconnected, try to reconnect.
+ *
+ *  Input:   h - database or statement handle
+ *
+ *  Returns: TRUE for success, FALSE otherwise
+ *
+ **************************************************************************/
+
+int MysqlReconnect(SV* h) {
+    D_imp_xxh(h);
+    imp_dbh_t* imp_dbh;
+    SV* sv;
+    HV* hv;
+    SV** svp;
+    SV* dbname;
+    SV* user;
+    SV* password;
+
+    if (DBIc_TYPE(imp_xxh) == DBIt_ST) {
+        imp_dbh = (imp_dbh_t*) DBIc_PARENT_COM(imp_xxh);
+	h = DBIc_PARENT_H(imp_xxh);
+    } else {
+        imp_dbh = (imp_dbh_t*) imp_xxh;
+    }
+
+    sv = DBIc_IMP_DATA(imp_dbh);
+    if (!sv  ||  !SvROK(sv)  ||  SvTYPE(SvRV(sv)) != SVt_PVHV) {
+        return FALSE;
+    }
+
+    hv = (HV*) SvRV(sv);
+    if (!(svp = hv_fetch(hv, "dsn", 3, FALSE))) {
+        return FALSE;
+    }
+    dbname = *svp;
+    if (!(svp = hv_fetch(hv, "user", 4, FALSE))) {
+        return FALSE;
+    }
+    user = *svp;
+    if (!(svp = hv_fetch(hv, "password", 8, FALSE))) {
+        return FALSE;
+    }
+    password = *svp;
+    return Login(h, imp_dbh, SvPV(dbname, na), SvPV(user, na),
+		 SvPV(password, na));
+}
+
