@@ -16,7 +16,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '2.009902';
+$VERSION = '2.0100';
 
 bootstrap DBD::mSQL $VERSION;
 
@@ -71,9 +71,11 @@ sub connect {
     # create a 'blank' dbh
     my $this;
 
-    if (!defined($this = DBI::_new_dbh($drh, { 'dsn' => $dsn,
-					       'user' => $username
-                                             }))) {
+    if (!defined($this = DBI::_new_dbh($drh, {},
+				       { 'dsn' => $dsn,
+					 'user' => $username,
+					 'password' => $password
+					 }))) {
 	return undef;
     }
 
@@ -94,6 +96,38 @@ sub data_sources {
     @dsn;
 }
 
+sub admin {
+    my($drh) = shift;
+    my($command) = shift;
+    my($dbname) = ($command eq 'createdb'  ||  $command eq 'dropdb') ?
+	shift : '';
+    my($host) = shift || '';
+    my ($user) = shift || '';
+    my ($password) = shift || '';
+    $drh->func(undef, $command, $dbname, $host, $user, $password,
+	       '_admin_internal');
+}
+
+sub _CreateDB {
+    my($drh) = shift;
+    my($host) = (@_ > 1) ? shift : undef;
+    my($dbname) = shift;
+    if (!$DBD::mSQL::QUIET) {
+	warn "'_CreateDB' is deprecated, use 'admin' instead";
+    }
+    $drh->func('createdb', $dbname, $host, 'admin');
+}
+
+sub _DropDB {
+    my($drh) = shift;
+    my($host) = (@_ > 1) ? shift : undef;
+    my($dbname) = shift;
+    if (!$DBD::mSQL::QUIET) {
+	warn "'DropDB' is deprecated, use 'admin' instead";
+    }
+    $drh->func('dropdb', $dbname, $host, 'admin');
+}
+
 
 package DBD::mSQL::db; # ====== DATABASE ======
 use strict;
@@ -154,15 +188,7 @@ sub ANSI2db {
     return $DBD::mSQL::db::ANSI2db{"$type"};
 }
 
-sub quote {
-    my ($self, $str) = @_;
-    if (!defined($str)) { return 'NULL'; }
-    $str =~ s/([\0\\\'])/\\$1/g;
-    "'$str'";
-}
-
-# Just a stub for backward compatibility; use is deprecated
-sub _ListFields($$) {
+sub listfields($$) {
     my ($self, $table) = @_;
     my ($sth);
     if (!($sth = $self->prepare("LISTFIELDS $table"))) {
@@ -174,14 +200,42 @@ sub _ListFields($$) {
     $sth;
 }
 
+sub _ListFields($$) {
+    my($self, $table) = @_;
+    if (!$DBD::mSQL::QUIET) {
+	warn "'_ListFields' is deprecated, use 'listfields' instead.";
+    }
+    $self->func($table, 'listfields');
+}
+
+sub admin {
+    my($dbh) = shift;
+    my($command) = shift;
+    my($dbname) = ($command eq 'createdb'  ||  $command eq 'dropdb') ?
+	shift : '';
+    $dbh->{'Driver'}->func($dbh, $command, $dbname, '', '', '',
+			   '_admin_internal');
+}
+
+
 package DBD::mSQL::st; # ====== STATEMENT ======
 use strict;
 
 # Just a stub for backward compatibility; use is deprecated
 sub _ListSelectedFields ($) {
+    if (!$DBD::mSQL::QUIET) {
+	warn "_ListSelectedFields is deprecated and superfluos";
+    }
     shift;
 }
 
+sub _NumRows ($) {
+    if (!$DBD::mSQL::QUIET) {
+	warn "_NumRows is deprecated, use \$sth->rows instead.";
+    }
+    shift->rows;
+}
+
 1;
 
 
@@ -201,15 +255,13 @@ Interface (DBI)
 
         or
 
-    $dsn = "DBI:mysql:$database:$hostname:$port";
-    if ($compression) { $dsn .= ";mysql_compression=1"; }
-    $dbh = DBI->connect($dsn, $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
+			$user, $password);
 
     @databases = DBD::mysql::dr->func( $hostname, '_ListDBs' );
     @tables = $dbh->func( '_ListTables' );
 
-    $sth = $dbh->prepare("LISTFIELDS $table");
-    $sth->execute;
+    $sth = $dbh->listfields($table);
     $sth->finish;
 
     $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
@@ -218,17 +270,36 @@ Interface (DBI)
     $numFields = $sth->{'NUM_OF_FIELDS'};
     $sth->finish;
 
-    $rc = $drh->func( $database, '_CreateDB' );
-    $rc = $drh->func( $host, $database, '_CreateDB' );
-    $rc = $drh->func( $database, '_DropDB' );
-    $rc = $drh->func( $host, $database, '_DropDB' );
+    $rc = $drh->func('createdb', $database, $host, $user, $password, 'admin');
+    $rc = $drh->func('dropdb', $database, $host, $user, $password, 'admin');
+    $rc = $drh->func('shutdown', $host, $user, $password, 'admin');
+    $rc = $drh->func('reload', $host, $user, $password, 'admin');
+
+    $rc = $dbh->func('createdb', $database, 'admin');
+    $rc = $dbh->func('dropdb', $database, 'admin');
+    $rc = $dbh->func('shutdown', 'admin');
+    $rc = $dbh->func('reload', 'admin');
+
+
+
+=head1 EXPERIMENTAL SOFTWARE
+
+This package contains experimental software and should *not* be used
+in a production environment. We are following the Linux convention and
+treat the "even" releases (1.18xx as of this writing, perhaps 1.20xx,
+1.22xx, ... in the future) as stable. Only bug or portability fixes
+will go into these releases.
+
+The "odd" releases (1.19xx as of this writing, perhaps 1.21xx, 1.23xx
+in the future) will be used for testing new features or other serious
+code changes.
 
 
 =head1 DESCRIPTION
 
 <DBD::mysql> and <DBD::mSQL> are the Perl5 Database Interface drivers for
 the mysql, mSQL 1.I<x> and mSQL 2.I<x> databases. The drivers are part
-of the I<mysql-modules> and I<Msql-modules> packages, respectively.
+of the I<Msql-Mysql-modules> package.
 
 
 =head2 Class Methods
@@ -248,12 +319,11 @@ of the I<mysql-modules> and I<Msql-modules> packages, respectively.
 
     use DBI;
 
-    $dsn = "DBI:mysql:$database;$options";
-    $dsn = "DBI:mysql:$database;$hostname;$options";
-    $dsn = "DBI:mysql:$database;$hostname:$port;$options";
-    $dsn = "DBI:mysql:$database;$hostname:$port;$options";
-
-    $dbh = DBI->connect($dsn, $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database", $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname",
+			$user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
+			$user, $password);
 
 A C<database> must always be specified.
 
@@ -266,30 +336,6 @@ you may explicitly state the port number to connect to in the C<hostname>
 argument, by concatenating the I<hostname> and I<port number> together
 separated by a colon ( C<:> ) character.
 
-Other options, possibly separated by semicolons, are:
-
-=over 8
-
-=item mysql_compression
-
-As of MySQL 3.22.3, a new feature is supported: If you use the option
-
-    mysql_compression=1
-
-then the communication between client and server will be compressed.
-
-=item mysql_socket
-
-As of MySQL 3.21.15, it is possible to choose the Unix socket that is
-used for connecting to the server. This is done, for example, with
-
-    mysql_socket=/dev/mysql
-
-Usually there's no need for this option, unless you are using another
-location for the socket than that built into the client.
-
-=back
-
 =back
 
 =head2 Private MetaData Methods
@@ -334,9 +380,30 @@ an empty list is returned.
       }
 
 
+=item B<listfields>
+
+    $sth = $dbh->func($table, 'listfields');
+
+The listfields method returns a statement handle that can be used for
+obtaining informations about the columns of $table. For example you
+may consult $sth->{'type'} for the column types, $sth->{'NUM_OF_FIELDS'}
+for the number of columns and so on. See L</STATEMENT HANDLES> above.
+The method is equivalent to
+
+    $sth = $dbh->prepare("LISTFIELDS $table");
+    $sth->execute;
+
+In particular this means that you should ensure that $sth goes out of
+scope or call $sth->finish before disconnecting from the database, as
+you will receive a warning otherwise.
+
+
 =item B<ListFields>
 
-Deprecated, see L</COMPATIBILITY ALERT> below.
+Deprecated, see L</COMPATIBILITY ALERT> below. Used to be equivalent
+to
+
+    $sth = $dbh->func($table, 'listfields');
 
 =item B<ListSelectedFields>
 
@@ -345,13 +412,74 @@ Deprecated, see L</COMPATIBILITY ALERT> below.
 =back
 
 
-=head2 Database Manipulation
+=head2 Server Administration
 
 =over 4
 
-=item B<CreateDB>
+=item admin
 
-=item B<DropDB>
+    $rc = $drh->func("createdb", $dbname, [host, user, password,], 'admin');
+    $rc = $drh->func("dropdb", $dbname, [host, user, password,], 'admin');
+    $rc = $drh->func("shutdown", [host, user, password,], 'admin');
+    $rc = $drh->func("reload", [host, user, password,], 'admin');
+
+      or
+
+    $rc = $dbh->func("createdb", $dbname, 'admin');
+    $rc = $dbh->func("dropdb", $dbname, 'admin');
+    $rc = $dbh->func("shutdown", 'admin');
+    $rc = $dbh->func("reload", 'admin');
+
+For server administration you need a server connection. For obtaining
+this connection you have two options: Either use a driver handle (drh)
+and supply the appropriate arguments (host, defaults localhost, user,
+defaults to '' and password, defaults to ''). A driver handle can be
+obtained with
+
+    $drh = DBI->install_driver('mSQL');
+
+Otherwise reuse the existing connection of a database handle (dbh).
+
+There's only one function available for administrative purposes, comparable
+to the m(y)sqladmin programs. The command being execute depends on the
+first argument:
+
+=over 8
+
+=item createdb
+
+Creates the database $dbname. Equivalent to "m(y)sqladmin create $dbname".
+
+=item dropdb
+
+Drops the database $dbname. Equivalent to "m(y)sqladmin drop $dbname".
+
+It should be noted that database deletion is
+I<not prompted for> in any way.  Nor is it undo-able from DBI.
+
+    Once you issue the dropDB() method, the database will be gone!
+
+These method should be used at your own risk.
+
+=item shutdown
+
+Silently shuts down the database engine. (Without prompting!)
+Equivalent to "m(y)sqladmin shutdown".
+
+=item reload
+
+Reloads the servers configuration files and/or tables. This can be particularly
+important if you modify access privileges or create new users.
+
+=back
+
+
+=item B<_CreateDB>
+
+=item B<_DropDB>
+
+
+These methods are deprecated, see L</COMPATIBILITY ALERT> below.!
 
     $rc = $drh->func( $database, '_CreateDB' );
     $rc = $drh->func( $database, '_DropDB' );
@@ -361,33 +489,13 @@ Deprecated, see L</COMPATIBILITY ALERT> below.
     $rc = $drh->func( $host, $database, '_CreateDB' );
     $rc = $drh->func( $host, $database, '_DropDB' );
 
-These two methods allow programmers to create and drop databases from
-DBI scripts. Since mSQL disallows the creation and deletion of
-databases over the network, these methods explicitly connect to the
-mSQL daemon running on the machine C<localhost> and execute these
-operations there.
-
-It should be noted that database deletion is I<not prompted for> in
-any way.  Nor is it undo-able from DBI.
-
-    Once you issue the dropDB() method, the database will be gone!
-
-These methods should be used at your own risk.
+These methods are equivalent to the admin method with "createdb" or
+"dropdb" commands, respectively. In particular note the warnings
+concerning the missing prompt for dropping a database!
 
 =back
 
 
-=head1 DATABASE HANDLES
-
-The DBD::mysql driver supports the following attributes of database
-handles (read only):
-
-    $infoString = $dbh->{'info'};
-    $threadId = $dbh->{'thread_id'};
-
-These correspond to mysql_info() and mysql_tread_id(), respectively.
-
-
 =head1 STATEMENT HANDLES
 
 The statement handles of DBD::mysql and DBD::mSQL support a number
@@ -439,7 +547,7 @@ header of table names together with all rows:
       }
       print "\n";
   }
-
+x
 For portable applications you should restrict yourself to attributes with
 capitalized or mixed case names. Lower case attribute names are private
 to DBD::mSQL and DBD::mysql. The attribute list includes:
@@ -526,124 +634,155 @@ MySQL uses &DBD::mysql::FIELD_TYPE_SHORT, &DBD::mysql::FIELD_TYPE_STRING etc.
 
 =head1 COMPATIBILITY ALERT
 
-As of version 0.70 DBD::mSQL has a new maintainer. Even more, the sources
-have been completely rewritten in August 1997, so it seemed apropriate
-to bump the version number: Incompatibilities are more than likely.
+Certain attributes methods have been declared obsolete or deprecated,
+partially because there names are agains DBI's naming conventions,
+partially because they are just superfluous or obsoleted by other methods.
 
-=head2 Recent changes:
+Obsoleted attributes and methods will be explicitly listed below. You cannot
+expect them to work in future versions, but they have not yet been scheduled
+for removal and currently they should be usable without any code modifications.
 
-=over 2
+Deprecated attributes and methods will currently issue a warning unless
+you set the variable $DBD::mSQL::QUIET to a true value. This will
+be the same for Msql-Mysql-modules 1.19xx and 1.20xx. They will be silently
+removed in 1.21xx.
 
-=item New connect method
-
-DBD::mSQL and DBD::mysql now use the new I<connect> method as introduced
-with DBI 0.83 or so. For compatibility reasons the old method still
-works, but the driver issues a warning when he detects use of the
-old version. There's no workaround, you must update your sources.
-(Sorry, but the change was in DBI, not in DBD::mysql and DBD::mSQL.)
-
-=item _ListFields returning statement handle
-
-As of Msql-modules 1.1805, the private functions
-
-    $dbh->func($table, "_ListFields");
-
-and
-
-    $sth->func("_ListSelectedFields");
-
-no longer return a simple hash, but a statement handle.
-(I<_ListSelectedFields> is a stub now which just returns $self.)
-This should usually not be visible, when your statement handle gets
-out of scope. However, if your database handle (C<$dbh> in the
-above example) disconnects, either because you explicitly disconnect
-or because he gets out of scope, and the statement handle is still
-active, DBI will issue a warning for active cursors being destroyed.
-
-The simple workaround is to execute C<$sth-E<gt>finish> or to ensure
-that C<$sth> gets out of scope before C<$dbh>. Sorry, but it was
-obvious nonsense to support two different things for accessing the
-basically same thing: A M(y)SQL result.
-
-=back
-
-The drivers do not conform to the current DBI specification in some minor
-points. For example, the private attributes I<is_num> or I<is_blob> have
-been written I<IS_NUM> and I<IS_BLOB>. For historical reasons we continue
-supporting the capitalized names, although the DBI specification now
-reserves capitalized names for standard names, mixed case for DBI and lower
-case for private attributes and methods.
-
-We currently consider anything not conforming to the DBI as deprecated.
-It is quite possible that we remove support of these deprecated names
-and methods in the future. In particular these includes:
+Here is a list of obsoleted attributes and/or methods:
 
 =over 4
 
-=item C<$sth-E<gt>func($table, '_ListSelectedFields')>
+=item _CreateDB
 
-highly deprecated, all attributes are directly accessible via the
-statement handle. For example instead of
+=item _DropDB
 
-  $ref = $sth->func($table, '_ListSelectedFields')
-  my @names = $ref->{'NAME'}
+deprecated, use
 
-you just do a
+    $drh->func("createdb", $dbname, $host, "admin")
+    $drh->func("dropdb", $dbname, $host, "admin")
 
-  my @names = @{$sth->{'NAME'}};
+=item _ListFields
 
-=item Capitalized attribute names
+deprecated, use
 
-Deprecated, should be replaced by the respective lower case names.
+    $dbh->func($table, "listfields")
+
+=item _ListSelectedFields
+
+deprecated, just use the statement handles for accessing the same attributes.
+
+=item _NumRows
+
+deprecated, use
+
+    $numRows = $sth->rows;
+
+=item IS_PRI_KEY
+
+=item IS_NOT_NULL
+
+=item IS_KEY
+
+=item IS_BLOB
+
+=item IS_NUM
+
+=item LENGTH
+
+=item MAXLENGTH
+
+=item NUMROWS
+
+=item NUMFIELDS
+
+=item RESULT
+
+=item TABLE
+
+=item TYPE
+
+All these statement handle attributes are obsolete. They can be simply
+replaced by just downcasing the attribute names. You should expect them
+to be deprecated as of Msql-Mysql-modules 1.1821. (Whenever that will
+be.)
 
 =back
 
 
-=head1 MULTITHREADING
-
-The multithreading capabilities of the Msql-Mysql-modules depend completely
-on the underlying C libraries: The modules are working with handle data
-only, no global variables are accessed or (to the best of my knowledge)
-thread unsafe functions are called. Thus DBD::mSQL and DBD::mysql are
-completely thread safe, if the C libraries thread safe and you don't
-share handles among threads.
-
-The obvious questions is: Are the C libraries thread safe? In the case of
-mSQL the answer is definitely "no". The C library has a concept of one
-single active connection at a time and that is not what threads like.
-
-In the case of MySQL the answer is "mostly" and, in theory, you should
-be able to get a "yes", if the C library is compiled for being thread
-safe (By default it isn't.) by passing the option -with-thread-safe-client
-to configure. See the section on I<How to make a threadsafe client> in
-the manual.
-
-
 =head1 BUGS
 
 The I<port> part of the first argument to the connect call is
-implemented in an unsafe way when using mSQL. In fact it is just
-stting the environment variable MSQL_TCP_PORT during the connect
-call. If another connect call uses another port and the handles
-are used simultaneously, they will interfere. I doubt that this
-will ever change.
+implemented in an unsafe way. In fact it never did more than set
+the environment variable MSQL_TCP_PORT during the connect call. If
+another connect call uses another port and the handles are used
+simultaneously, they will interfere. In a future version this
+behaviour will hoefully change, depending on David and Monty. :-)
 
-Msql-2.0.4 and 2.0.4.1 contain a bug that makes ORDER BY and hence
-the test script C<t/40bindparam> fail. To verify, if this is the
-case for you, do a
+The func method call on a driver handle seems to be undocumented in
+the DBI manpage. DBD::mSQL has func methods on driverhandles, database
+handles, and statement handles. What gives?
 
-	cd Msql
-	perl -w -I../blib/lib -I../blib/arch t/40bindparam.t
+Please speak up now (June 1997) if you encounter additional bugs. I'm
+still learning about the DBI API and can neither judge the quality of
+the code presented here nor the DBI compliancy. But I'm intending to
+resolve things quickly as I'd really like to get rid of the multitude
+of implementations ASAP.
 
-If something is wrong, the script ought to print a number of id's and
-names. If the id's aren't in order, it is likely, that your mSQL has
-a bug. See the INSTALL file for a patch.
+When running "make test", you will notice that some test scripts fail.
+This is due to bugs in the respective databases, not in the DBI drivers:
 
+=over 4
+
+=item Nulls
+
+mSQL seems to have problems with NULL's: The following fails with
+mSQL 2.0.1 running on a Linux 2.0.30 machine:
+
+    [joe@laptop Msql-modules-1.18]$ msql test
+    Welcome to the miniSQL monitor.  Type \h for help.
+    mSQL > CREATE TABLE foo (id INTEGER, name CHAR(6))\g
+    Query OK.  1 row(s) modified or retrieved.
+    mSQL > INSERT INTO foo VALUES (NULL, 'joe')\g
+    Query OK.  1 row(s) modified or retrieved.
+    mSQL > SELECT * FROM foo WHERE id = NULL\g
+    Query OK.  0 row(s) modified or retrieved.
+    +----------+------+
+    | id       | name |
+    +----------+------+
+    +----------+------+
+    mSQL > 
+
+=item Blanks
+
+mysql has problems with Blanks on the right side of string fields: They
+get chopped of. (Tested with mysql 3.20.25 on a Linux 2.0.30 machine.)
+
+    [joe@laptop Msql-modules-1.18]$ mysql test
+    Welcome to the mysql monitor.  Commands ends with ; or \g.
+    Type 'help' for help.
+    mysql> CREATE TABLE foo (id INTEGER, bar CHAR(8));
+    Query OK, 0 rows affected (0.10 sec)
+    mysql> INSERT INTO foo VALUES (1, ' a b c ');
+    Query OK, 1 rows affected (0.00 sec)
+    mysql> SELECT * FROM foo;
+    1 rows in set (0.19 sec)
+    +------+--------+
+    | id   | bar    |
+    +------+--------+
+    |    1 |  a b c |
+    +------+--------+
+    mysql> quit;
+    [joe@laptop Msql-modules-1.18]$ mysqldump test foo
+
+    [deleted]
+
+    INSERT INTO foo VALUES (1,' a b c');
+
+=back
 
 =head1 AUTHOR
 
 B<DBD::mSQL> has been primarily written by Alligator Descartes
-(I<descarte@arcana.co.uk>), who has been aided and abetted by Gary
+(I<descarte@hermetica.com>), who has been aided and abetted by Gary
 Shea, Andreas Koenig and Tim Bunce amongst others. Apologies if your
 name isn't listed, it probably is in the file called
 'Acknowledgments'. As of version 0.80 the maintainer is Andreas Knig.
@@ -665,36 +804,7 @@ current version and all credits and copyright notices are retained (
 the I<AUTHOR> and I<COPYRIGHT> sections ).  Requests for other
 distribution rights, including incorporation into commercial products,
 such as books, magazine articles or CD-ROMs should be made to
-Alligator Descartes <I<descarte@arcana.co.uk>>.
-
-
-=head1 MAILING LIST SUPPORT
-
-This module is maintained and supported on a mailing list,
-
-    msql-mysql-modules@tcx.se
-
-To subscribe to this list, send a mail with the words
-
-    subscribe msql-mysql-modules
-
-or
-
-    subscribe msql-mysql-modules-digest
-
-in the first line of the body to mdomo@tcx.se. A mailing list archive is
-in preparation.
-
-Additionally you might try the dbi-user mailing list for questions about
-DBI and its modules in general. Subscribe via
-
-    http://www.fugue.com/dbi
-
-Mailing list archives are at
-
-     http://www.rosat.mpe-garching.mpg.de/mailing-lists/PerlDB-Interest/
-     http://outside.organic.com/mail-archives/dbi-users/
-     http://www.coe.missouri.edu/~faq/lists/dbi.html
+Alligator Descartes <I<descarte@hermetica.com>>.
 
 
 =head1 ADDITIONAL DBI INFORMATION
@@ -702,7 +812,7 @@ Mailing list archives are at
 Additional information on the DBI project can be found on the World
 Wide Web at the following URL:
 
-    http://www.arcana.co.uk/technologia/perl/DBI
+    http://www.hermetica.com/technologia/perl/DBI
 
 where documentation, pointers to the mailing lists and mailing list
 archives and pointers to the most current versions of the modules can
